<?php
/**
 * Do not load this file directly
 *
 * @since 1.0.0
 */
if ( ! defined( 'ABSPATH' ) ) {
	die( '-1' );
}

/**
 * WPJ_OpenGraph class.
 *
 * @since 0.1.0
 */
class WPJ_OpenGraph {

	/**
	 * If no instance of WPJ_OpenGraph has been made then let's make it.
	 *
	 * @var object $instance
	 * @access private
	 * @static
	 *
	 * @since 0.1.0
	 */
	private static $instance = null;

	/**
	 * Kick off the class if it hasn't been instantiated.
	 *
	 * @return Singleton The Singleton instance of this class.
	 * @access public
	 * @static
	 *
	 * @since 0.1.0
	 */
	public static function get_instance() {
		if ( ! self::$instance ) {
			self::$instance = new WPJ_OpenGraph;
		}
		return self::$instance;
	}

	/**
	 * Init function.
	 *
	 * @since 0.1.0
	 */
	public function init() {
		add_action( 'wp_head', array( $this, 'open_graph' ), 1 );
		add_action( 'wpj_opengraph', array( $this, 'locale' ), 1 );
		add_action( 'wpj_opengraph', array( $this, 'type' ), 5 );
		add_action( 'wpj_opengraph', array( $this, 'og_title' ), 10 );
		add_action( 'wpj_opengraph', array( $this, 'description' ), 11 );
		add_action( 'wpj_opengraph', array( $this, 'url' ), 12 );
		add_action( 'wpj_opengraph', array( $this, 'site_name' ), 13 );
		add_action( 'wpj_opengraph', array( $this, 'app_id' ), 20 );
		add_action( 'wpj_opengraph', array( $this, 'article_author_facebook' ), 15 );
		add_action( 'wpj_opengraph', array( $this, 'category' ), 17 );
		add_action( 'wpj_opengraph', array( $this, 'publish_date' ), 19 );
		add_action( 'wpj_opengraph', array( $this, 'image' ), 30 );
	}

	/**
	 * Main OpenGraph output.
	 */
	public function open_graph() {
		do_action( 'wpj_opengraph' );
	}

	/**
	 * Internal function to output FB tags.
	 *
	 * @param string $property Property attribute value.
	 * @param string $content  Content attribute value.
	 * @return boolean
	 */
	public function og_tag( $property, $content ) {
		$og_property = str_replace( ':', '_', $property );

		if ( empty( $content ) ) {
			return false;
		}

		echo '<meta property="', esc_attr( $property ), '" content="', esc_attr( $content ), '" />', "\n";

		return true;
	}

	/**
	 * Output the locale.
	 *
	 * We do some conversions to make sure the proper Facebook locale is outputted.
	 *
	 * @see  http://www.facebook.com/translations/FacebookLocales.xml for the list of supported locales
	 * @link https://developers.facebook.com/docs/reference/opengraph/object-type/article/
	 * @param bool $echo Whether to echo or return the locale.
	 * @return string $locale
	 */
	public function locale() {
		$locale = get_locale();

		// Catch some weird locales served out by WP that are not easily doubled up.
		$fix_locales = array(
			'ca' => 'ca_ES',
			'en' => 'en_US',
			'el' => 'el_GR',
			'et' => 'et_EE',
			'ja' => 'ja_JP',
			'sq' => 'sq_AL',
			'uk' => 'uk_UA',
			'vi' => 'vi_VN',
			'zh' => 'zh_CN',
		);

		if ( isset( $fix_locales[ $locale ] ) ) {
			$locale = $fix_locales[ $locale ];
		}

		// Convert locales like "es" to "es_ES".
		if ( strlen( $locale ) === 2 ) {
			$locale = strtolower( $locale ) . '_' . strtoupper( $locale );
		}

		// These are the locales FB supports.
		$fb_valid_fb_locales = array(
			'af_ZA', // Afrikaans.
			'ak_GH', // Akan.
			'am_ET', // Amharic.
			'ar_AR', // Arabic.
			'as_IN', // Assamese.
			'ay_BO', // Aymara.
			'az_AZ', // Azerbaijani.
			'be_BY', // Belarusian.
			'bg_BG', // Bulgarian.
			'bp_IN', // Bhojpuri.
			'bn_IN', // Bengali.
			'br_FR', // Breton.
			'bs_BA', // Bosnian.
			'ca_ES', // Catalan.
			'cb_IQ', // Sorani Kurdish.
			'ck_US', // Cherokee.
			'co_FR', // Corsican.
			'cs_CZ', // Czech.
			'cx_PH', // Cebuano.
			'cy_GB', // Welsh.
			'da_DK', // Danish.
			'de_DE', // German.
			'el_GR', // Greek.
			'en_GB', // English (UK).
			'en_PI', // English (Pirate).
			'en_UD', // English (Upside Down).
			'en_US', // English (US).
			'em_ZM',
			'eo_EO', // Esperanto.
			'es_ES', // Spanish (Spain).
			'es_LA', // Spanish.
			'es_MX', // Spanish (Mexico).
			'et_EE', // Estonian.
			'eu_ES', // Basque.
			'fa_IR', // Persian.
			'fb_LT', // Leet Speak.
			'ff_NG', // Fulah.
			'fi_FI', // Finnish.
			'fo_FO', // Faroese.
			'fr_CA', // French (Canada).
			'fr_FR', // French (France).
			'fy_NL', // Frisian.
			'ga_IE', // Irish.
			'gl_ES', // Galician.
			'gn_PY', // Guarani.
			'gu_IN', // Gujarati.
			'gx_GR', // Classical Greek.
			'ha_NG', // Hausa.
			'he_IL', // Hebrew.
			'hi_IN', // Hindi.
			'hr_HR', // Croatian.
			'hu_HU', // Hungarian.
			'ht_HT', // Haitian Creole.
			'hy_AM', // Armenian.
			'id_ID', // Indonesian.
			'ig_NG', // Igbo.
			'is_IS', // Icelandic.
			'it_IT', // Italian.
			'ik_US',
			'iu_CA',
			'ja_JP', // Japanese.
			'ja_KS', // Japanese (Kansai).
			'jv_ID', // Javanese.
			'ka_GE', // Georgian.
			'kk_KZ', // Kazakh.
			'km_KH', // Khmer.
			'kn_IN', // Kannada.
			'ko_KR', // Korean.
			'ks_IN', // Kashmiri.
			'ku_TR', // Kurdish (Kurmanji).
			'ky_KG', // Kyrgyz.
			'la_VA', // Latin.
			'lg_UG', // Ganda.
			'li_NL', // Limburgish.
			'ln_CD', // Lingala.
			'lo_LA', // Lao.
			'lt_LT', // Lithuanian.
			'lv_LV', // Latvian.
			'mg_MG', // Malagasy.
			'mi_NZ', // Maori.
			'mk_MK', // Macedonian.
			'ml_IN', // Malayalam.
			'mn_MN', // Mongolian.
			'mr_IN', // Marathi.
			'ms_MY', // Malay.
			'mt_MT', // Maltese.
			'my_MM', // Burmese.
			'nb_NO', // Norwegian (bokmal).
			'nd_ZW', // Ndebele.
			'ne_NP', // Nepali.
			'nl_BE', // Dutch (Belgie).
			'nl_NL', // Dutch.
			'nn_NO', // Norwegian (nynorsk).
			'nr_ZA', // Southern Ndebele.
			'ns_ZA', // Northern Sotho.
			'ny_MW', // Chewa.
			'om_ET', // Oromo.
			'or_IN', // Oriya.
			'pa_IN', // Punjabi.
			'pl_PL', // Polish.
			'ps_AF', // Pashto.
			'pt_BR', // Portuguese (Brazil).
			'pt_PT', // Portuguese (Portugal).
			'qc_GT', // Quiché.
			'qu_PE', // Quechua.
			'qr_GR',
			'qz_MM', // Burmese (Zawgyi).
			'rm_CH', // Romansh.
			'ro_RO', // Romanian.
			'ru_RU', // Russian.
			'rw_RW', // Kinyarwanda.
			'sa_IN', // Sanskrit.
			'sc_IT', // Sardinian.
			'se_NO', // Northern Sami.
			'si_LK', // Sinhala.
			'su_ID', // Sundanese.
			'sk_SK', // Slovak.
			'sl_SI', // Slovenian.
			'sn_ZW', // Shona.
			'so_SO', // Somali.
			'sq_AL', // Albanian.
			'sr_RS', // Serbian.
			'ss_SZ', // Swazi.
			'st_ZA', // Southern Sotho.
			'sv_SE', // Swedish.
			'sw_KE', // Swahili.
			'sy_SY', // Syriac.
			'sz_PL', // Silesian.
			'ta_IN', // Tamil.
			'te_IN', // Telugu.
			'tg_TJ', // Tajik.
			'th_TH', // Thai.
			'tk_TM', // Turkmen.
			'tl_PH', // Filipino.
			'tl_ST', // Klingon.
			'tn_BW', // Tswana.
			'tr_TR', // Turkish.
			'ts_ZA', // Tsonga.
			'tt_RU', // Tatar.
			'tz_MA', // Tamazight.
			'uk_UA', // Ukrainian.
			'ur_PK', // Urdu.
			'uz_UZ', // Uzbek.
			've_ZA', // Venda.
			'vi_VN', // Vietnamese.
			'wo_SN', // Wolof.
			'xh_ZA', // Xhosa.
			'yi_DE', // Yiddish.
			'yo_NG', // Yoruba.
			'zh_CN', // Simplified Chinese (China).
			'zh_HK', // Traditional Chinese (Hong Kong).
			'zh_TW', // Traditional Chinese (Taiwan).
			'zu_ZA', // Zulu.
			'zz_TR', // Zazaki.
		);

		// Check to see if the locale is a valid FB one, if not, use en_US as a fallback.
		if ( ! in_array( $locale, $fb_valid_fb_locales, true ) ) {
			$locale = strtolower( substr( $locale, 0, 2 ) ) . '_' . strtoupper( substr( $locale, 0, 2 ) );
			if ( ! in_array( $locale, $fb_valid_fb_locales, true ) ) {
				$locale = 'en_US';
			}
		}

		$this->og_tag( 'og:locale', $locale );
	}

	/**
	 * Output the OpenGraph type.
	 *
	 * @link https://developers.facebook.com/docs/reference/opengraph/object-type/object/
	 * @param boolean $echo Whether to echo or return the type.
	 * @return string $type
	 */
	public function type() {
		if ( is_front_page() || is_home() ) {
			$type = 'website';
		}
		elseif ( is_singular() ) {
			$type = 'article';
		}
		else {
			// We use "object" for archives etc. as article doesn't apply there.
			$type = 'object';
		}

		if ( is_string( $type ) && $type !== '' ) {
			$this->og_tag( 'og:type', $type );
		}
	}

	/**
	 * Outputs the title as OpenGraph title.
	 *
	 * @link https://developers.facebook.com/docs/reference/opengraph/object-type/article/
	 * @param bool $echo Whether or not to echo the output.
	 * @return string|boolean
	 */
	public function og_title() {
		if ( is_front_page() || is_home() ) {
			$title = get_bloginfo( 'name' );
		}
		elseif ( is_singular() ) {
			$title = get_the_title();
		}
		elseif ( is_category() ) {
			$title = single_cat_title( '', false );
		}
		elseif ( is_tag() ) {
			$title = single_tag_title( '', false );
		}
		elseif ( is_tax() ) {
			$title = single_term_title( '', false );
		}
		else {
			$title = '';
		}

		if ( is_string( $title ) && $title !== '' ) {
			$this->og_tag( 'og:title', $title );
		}
	}

	/**
	 * Output the OpenGraph description.
	 *
	 * @param bool $echo Whether to echo or return the description.
	 * @return string $ogdesc
	 */
	public function description() {
		global $post;
		$ogdesc   = '';

		if ( is_front_page() || is_home() ) {
			$ogdesc = get_bloginfo( 'description' );
		}
		elseif ( is_singular() ) {
			if ( has_excerpt( $post->ID ) ) {
				$ogdesc = str_replace( '[&hellip;]', '&hellip;', wp_strip_all_tags( get_the_excerpt() ) );
			} else {
				$ogdesc = trim( str_replace( "\r\n", ' ' , substr( strip_tags( $post->post_content ), 0, 300 ) ) );
			}
		}
		elseif ( is_category() || is_tag() || is_tax() ) {
			$ogdesc = wp_strip_all_tags( term_description() );
		}

		// Strip shortcodes if any.
		$ogdesc = strip_shortcodes( $ogdesc );

		if ( is_string( $ogdesc ) && $ogdesc !== '' ) {
			$this->og_tag( 'og:description', $ogdesc );
		}
	}

	/**
	 * Outputs the canonical URL as OpenGraph URL, which consolidates likes and shares.
	 *
	 * @link https://developers.facebook.com/docs/reference/opengraph/object-type/article/
	 * @return boolean
	 */
	public function url() {
		$canonical = false;

		if ( is_singular() ) {
			$obj       = get_queried_object();
			$canonical = get_permalink( $obj->ID );
		}
		else {
			if ( is_search() ) {
				$search_query = get_search_query();
				// Regex catches case when /search/page/N without search term is itself mistaken for search term. R.
				if ( ! empty( $search_query ) && ! preg_match( '|^page/\d+$|', $search_query ) ) {
					$canonical = get_search_link();
				}
			}
			elseif ( is_front_page() ) {
				$canonical = home_url();
			}
			elseif ( is_home() ) {
				$posts_page_id = get_option( 'page_for_posts' );
				$canonical = get_permalink( $posts_page_id );
			}
			elseif ( is_tax() || is_tag() || is_category() ) {
				$term = get_queried_object();
				if ( ! empty( $term ) ) {
					$term_link = get_term_link( $term, $term->taxonomy );
				}

				if ( ! is_wp_error( $term_link ) ) {
					$canonical = $term_link;
				}

			}
			elseif ( is_post_type_archive() ) {
				$post_type = get_query_var( 'post_type' );
				$canonical = get_post_type_archive_link( $post_type );
			}
			elseif ( is_author() ) {
				$canonical = get_author_posts_url( get_query_var( 'author' ), get_query_var( 'author_name' ) );
			}
			elseif ( is_archive() ) {
				if ( is_date() ) {
					if ( is_day() ) {
						$canonical = get_day_link( get_query_var( 'year' ), get_query_var( 'monthnum' ), get_query_var( 'day' ) );
					}
					elseif ( is_month() ) {
						$canonical = get_month_link( get_query_var( 'year' ), get_query_var( 'monthnum' ) );
					}
					elseif ( is_year() ) {
						$canonical = get_year_link( get_query_var( 'year' ) );
					}
				}
			}
		}

		$url = $canonical;

		if ( is_string( $url ) && $url !== '' ) {
			$this->og_tag( 'og:url', esc_url( $url ) );
		}
	}

	/**
	 * Output the site name straight from the blog info.
	 */
	public function site_name() {
		$name = get_bloginfo( 'name' );
		if ( is_string( $name ) && $name !== '' ) {
			$this->og_tag( 'og:site_name', $name );
		}
	}

	/**
	 * Outputs the Facebook app_id.
	 *
	 * @link https://developers.facebook.com/docs/reference/opengraph/object-type/article/
	 * @return void
	 */
	public function app_id() {
		$app_id = $this->get_option( 'wpj_fb_appid' );
		if ( $app_id !== '' ) {
			$this->og_tag( 'fb:app_id', absint( $app_id ) );
		}
	}

	/**
	 * Outputs the authors FB page.
	 *
	 * @link https://developers.facebook.com/blog/post/2013/06/19/platform-updates--new-open-graph-tags-for-media-publishers-and-more/
	 * @link https://developers.facebook.com/docs/reference/opengraph/object-type/article/
	 * @return boolean
	 */
	public function article_author_facebook() {
		if ( ! is_singular() || is_front_page() ) {
			return false;
		}

		$facebook = get_the_author_meta( 'facebook', $GLOBALS['post']->post_author );
		if ( $facebook && ( is_string( $facebook ) && $facebook !== '' ) ) {
			$this->og_tag( 'article:author', $facebook );
		}
	}

	/**
	 * Output the article category as an article:section tag.
	 *
	 * @link https://developers.facebook.com/docs/reference/opengraph/object-type/article/
	 * @return boolean;
	 */
	public function category() {
		if ( ! is_singular() || is_front_page() ) {
			return false;
		}

		$post = get_post();
		if ( ! $post ) {
			return false;
		}

		$terms = get_the_category( $post->ID );
		if ( ! is_wp_error( $terms ) && is_array( $terms ) && ! empty( $terms ) ) {
			// We can only show one section here, so we take the first one.
			$term = reset( $terms );
			$this->og_tag( 'article:section', $term->name );
		}
	}

	/**
	 * Output the article publish and last modification date.
	 *
	 * @link https://developers.facebook.com/docs/reference/opengraph/object-type/article/
	 * @return boolean;
	 */
	public function publish_date() {
		if ( ! is_singular() || is_front_page() ) {
			return false;
		}

		$pub = get_the_time( DATE_W3C );
		$this->og_tag( 'article:published_time', $pub );

		$mod = get_the_modified_time( DATE_W3C );
		if ( $mod !== $pub ) {
			$this->og_tag( 'article:modified_time', $mod );
			$this->og_tag( 'og:updated_time', $mod );
		}
	}

	/**
	 * Output the article image.
	 */
	public function image() {
		$image_url = false;

		switch ( true ) {
			case is_front_page():
				$post_id   = get_option('page_on_front');
				$image_url = $this->get_image_by_id( $post_id );
				break;
			case is_home():
				$post_id   = get_option( 'page_for_posts' );
				$image_url = $this->get_image_by_id( $post_id );
				break;
			case is_singular():
				$post      = get_post();
				$post_id   = $post->ID;
				$image_url = $this->get_image_by_id( $post_id );
				break;
			case is_category():
			case is_tag():
			case is_tax():
			case is_post_type_archive():
				$image_url = $this->get_fallback_image();
		}

		if ( is_string( $image_url ) && $image_url !== '' ) {
			$this->og_tag( 'og:image', esc_url( $image_url ) );

			// Add secure URL if detected.
			if ( strpos( $image_url, 'https://' ) === 0 ) {
				$this->og_tag( 'og:image:secure_url', esc_url( $image_url ) );
			}
		}
	}

	/**
	 * Get the images by ID to set the og:image.
	 *
	 * @since  0.1.1
	 * @return Image url
	 */
	public function get_image_by_id( $post_id ) {
		if ( has_post_thumbnail( $post_id ) ) {
			$attachment_id = get_post_thumbnail_id( $post_id );
			$thumbnail_src = wp_get_attachment_image_src( $attachment_id, 'large' );
			$image_src     = reset( $thumbnail_src );
			return $image_src;
		} else {
			return $this->get_fallback_image();
		}
	}

	/**
	 * Get fallback image to set the og:image.
	 *
	 * @since  0.1.1
	 * @return Image url
	 */
	public function get_fallback_image() {
		return $this->get_option( 'wpj_ogp_image' );
	}

	/**
	 * Wrapper function around cmb2_get_option
	 *
	 * @since  0.1.0
	 * @param  string $key     Options array key
	 * @param  mixed  $default Optional default value
	 * @return mixed           Option value
	 */
	public function get_option( $key = '', $default = false ) {
		if ( function_exists( 'cmb2_get_option' ) ) {
			// Use cmb2_get_option as it passes through some key filters.
			return cmb2_get_option( 'wpj-opengraph-settings', $key, $default );
		}
		// Fallback to get_option if CMB2 is not loaded yet.
		$opts = get_option( 'wpj-opengraph-settings', $default );
		$val = $default;
		if ( 'all' == $key ) {
			$val = $opts;
		} elseif ( is_array( $opts ) && array_key_exists( $key, $opts ) && false !== $opts[ $key ] ) {
			$val = $opts[ $key ];
		}
		return $val;
	}
	
}